(function(global) {
	var baseUrl = 'http://52.203.165.173/brasileirao/standings';
	// var baseUrl = 'http://localhost:3000/standings'; //dev

	/**
	 * Recover data for the championship's standings. Returns data from cache
	 * first and update it later async. If there is no cached data, waits for
	 * the arrival of data
	 * @param  {Function} callback Callback called whenever data is available or with error status
	 */
	function recoverAndUpdateData (callback) {
		// Try cache first and then show up-to-date info later
		var recoveredData, lastFetch;
		try {
			recoveredData = global.localStorage.getItem('brasileirao-standings');
		} catch (e) {
			displayError(e);
		}

		if (recoveredData) {
			recoveredData = JSON.parse(recoveredData);
			lastFetch = recoveredData.timestamp;
			callback(null, recoveredData);
		}

		if (!(lastFetch && Date.now() < lastFetch + 60*60*1000)) { // No updates for 1 hour
			var request = new XMLHttpRequest(),
				endpoint = baseUrl + '?t=' + Date.now(); // Cache-buster timestamp

			request.open('GET', endpoint, true);
			request.onload = function() {
				if (this.status >= 200 && this.status < 400) {
					// Save for later
					var data = JSON.parse(this.response);
					if (data.standing && data.standing.length == 20) {
						data.timestamp = Date.now();
						try {
							global.localStorage.setItem('brasileirao-standings', JSON.stringify(data));
						} catch (e) {
							displayError('Error caching maps data');
						}
						if (!recoveredData) {
							callback(null, data);
						}
					} else {
						displayError('Invalid data received! ' + JSON.stringify(data));
					}
				} else {
					displayError('XHR Status ' + this.status);
					if (!recoveredData) {
						callback('ERROR: Status ' + this.status);
					}
				}
			};
			request.onerror = function() {
				displayError('XHR ERROR!');
				if (!recoveredData) {
					callback('ERROR: XHR ERROR!');
				}
			};
			request.ontimeout = function() {
				displayError('XHR timeout!')	;
				if (!recoveredData) {
					callback('ERROR: XHR Timeout!');
				}
			};
			request.timeout = 2000; //2s
			request.send();
		}
	}

	function displayError(err) {
		console.error(err);
	}

	global.fetchStandings = recoverAndUpdateData;
})(window);
